@@ -138,6 +138,7 @@ import atexit
 import datetime
 import math
 import os
+import errno
 import re
 import shlex
 import shutil
@@ -481,7 +482,7 @@ class Device:
         """
         cmdResult = self.adb("shell", "su --help || echo SLFAILED", check=False)
 
-        if ("Magisk" in cmdResult.stdout):
+        if "Magisk" in cmdResult.stdout:
             print("    Warning: Magisk rooted devices are not officially supported by this product.")
 
         return (cmdResult.returncode == 0) and ("SLFAILED" not in cmdResult.stdout)
@@ -1335,7 +1336,7 @@ def run_gatord_interactive():
     input("\nWaiting for data capture ...")
 
 
-# pylint: disable-msg=too-many-locals,too-many-positional-arguments
+# pylint: disable-msg=too-many-locals,too-many-arguments,too-many-positional-arguments
 def run_gatord_headless(device: Device, package: str, outputName: str, timeout: int, activity: str, activityArgs: str, enableGpuTimeline: str):
     """
     Run gatord for a headless capture session.
@@ -1443,7 +1444,7 @@ def exit_handler(device: Device, args: ap.Namespace):
     device.adb_quiet("shell", "pkill", "gatord")
     device.adb_run_as(args.package, "pkill", "gatord", quiet=True)
 
-    if args.enableGpuTimeline != "no":
+    if args.enableGpuTimeline and args.enableGpuTimeline != "no":
         try:
             remove_gpu_timeline_layer(device, args)
         except sp.CalledProcessError as e:
@@ -1699,7 +1700,14 @@ def parse_cli(parser: ap.ArgumentParser):
             if os.path.isfile(args.headless):
                 os.remove(args.headless)
             else:
-                shutil.rmtree(args.headless)
+                try:
+                    shutil.rmtree(args.headless)
+                except PermissionError as err:
+                    if err.errno == errno.EACCES:
+                        print("Capture {0} is currently open by another process and cannot be overwritten.".format(args.headless))
+                    else:
+                        print("{0}".format(err))
+                    raise err
 
     # Ensure the lwi output path is absolute
     # Use a default path if the user hasn't specified it
@@ -1814,7 +1822,10 @@ def ensure_lwi_output_path_usable(abs_lwi_out_dir: str, overwrite: bool):
         if os.path.isfile(abs_lwi_out_dir):
             os.remove(abs_lwi_out_dir)
         else:
-            shutil.rmtree(abs_lwi_out_dir)
+            try:
+                shutil.rmtree(abs_lwi_out_dir)
+            except PermissionError as err:
+                print("{0}".format(err))
 
     if not os.path.exists(abs_lwi_out_dir):
         try:
@@ -1865,6 +1876,9 @@ def main() -> int:
         parser.print_usage()
         print("{0}".format(err))
         return 4
+    except PermissionError as _err:
+        # Just silently ignore exception. Exception was handled before.
+        return 5
 
     if args.verbose:
         global DEBUG_GATORD  # pylint: disable=global-statement
@@ -2046,7 +2060,7 @@ def main() -> int:
                 args.glesLayerLibPath = glesLayerLibPath
                 enable_gles_debug_layer(device, args)
 
-    if args.enableGpuTimeline != "no":
+    if args.enableGpuTimeline and args.enableGpuTimeline != "no":
         timelineLayerLibPath = args.timelineLayerLibPath
         isAuto = args.enableGpuTimeline == "auto"
         isValidLibFile = True
@@ -2104,7 +2118,7 @@ def main() -> int:
             if "gles" in args.lwiApi:
                 disable_gles_debug_layer(device, args)
 
-        if args.enableGpuTimeline != "no":
+        if args.enableGpuTimeline and args.enableGpuTimeline != "no":
             remove_gpu_timeline_layer(device, args)
 
         atexit.unregister(exit_handler)
