@@ -1,6 +1,7 @@
-/* Copyright (C) 2021-2024 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2021-2025 by Arm Limited. All rights reserved. */
 #pragma once
 
+#include "Logging.h"
 #include "agents/agent_worker_base.h"
 #include "agents/ext_source/ext_source_connection.h"
 #include "agents/spawn_agent.h"
@@ -11,8 +12,7 @@
 
 #include <cerrno>
 #include <memory>
-#include <stdexcept>
-#include <variant>
+#include <unordered_set>
 
 #include <boost/asio/bind_executor.hpp>
 #include <boost/asio/dispatch.hpp>
@@ -69,7 +69,43 @@ namespace agents {
 
         boost::asio::io_context::strand strand;
         ExternalSource & external_source;
-        std::map<ipc::annotation_uid_t, boost::asio::posix::stream_descriptor> external_source_pipes {};
+        std::map<ipc::annotation_uid_t, boost::asio::posix::stream_descriptor> external_source_pipes;
+
+        /**
+         * UIDs of external source pipes which have been closed. This helps
+         * avoid errors should those UIDs be encountered in the future. This
+         * will only work if UIDs are unique (which is currently the case).
+         */
+        std::unordered_set<ipc::annotation_uid_t> closed_external_source_pipes;
+        ipc::msg_gpu_timeline_configuration_t gpu_timeline_config;
+
+        /**
+         * @brief Closes a source pipe with a given UID while managing object
+         * data structures as appropriate.
+         * @return Whether the source pipe was found. If false, nothing has been
+         * changed.
+         */
+        bool close_external_source_pipe(ipc::annotation_uid_t uid)
+        {
+            // UIDs are eternally unique (assert rather than error as this
+            // assumption is mainly to avoid misleading errors)
+            assert(closed_external_source_pipes.count(uid) == 0);
+
+            // close this end first
+            auto it = external_source_pipes.find(uid);
+            if (it == external_source_pipes.end()) {
+                return false;
+            }
+
+            // close the write end
+            it->second.close();
+
+            // and remove from the map
+            external_source_pipes.erase(it);
+            closed_external_source_pipes.insert(uid);
+
+            return true;
+        }
 
         /** @return A continuation that requests the remote target to shutdown */
         auto cont_shutdown()
@@ -99,7 +135,11 @@ namespace agents {
                    });
         }
 
-        /** @return A continuation that closes the connection due to a write error on this end */
+        /**
+         * @return A continuation that closes the connection. This is either due
+         * to a write error on this end, or is simply on request (for example,
+         * because Gator is closing down).
+         */
         auto cont_close_annotation_uid(ipc::annotation_uid_t uid)
         {
             using namespace async::continuations;
@@ -108,19 +148,14 @@ namespace agents {
 
             return start_on(strand) //
                  | then([st, uid]() -> polymorphic_continuation_t<> {
-                       // close this end first
-                       auto it = st->external_source_pipes.find(uid);
-                       if (it == st->external_source_pipes.end()) {
+                       bool source_pipe_found = st->close_external_source_pipe(uid);
+
+                       if (!source_pipe_found) {
                            return {};
                        }
 
-                       // close the write end
-                       it->second.close();
-
-                       // and remove from the map
-                       st->external_source_pipes.erase(it);
-
-                       // close the external source pipe
+                       // close other end of the external source pipe
+                       // Note that msg_annotation_close_conn_t is shared with Timeline
                        return st->sink().async_send_message(ipc::msg_annotation_close_conn_t {uid}, use_continuation)
                             | then([st](auto const & ec, auto const & /*msg*/) -> polymorphic_continuation_t<> {
                                   if (ec) {
@@ -139,14 +174,28 @@ namespace agents {
         }
 
         /** Handle the 'ready' IPC message variant. The agent is ready. */
-        void cont_on_recv_message(ipc::msg_ready_t const & /*message*/)
+        auto cont_on_recv_message(ipc::msg_ready_t const & /*message*/)
         {
+            using namespace async::continuations;
+
             LOG_DEBUG("Received ready message.");
 
             // transition state
             if (transition_state(state_t::ready)) {
                 LOG_DEBUG("ext_source agent is now ready");
             }
+
+            return start_on(strand) //
+                 | sink().async_send_message(gpu_timeline_config, use_continuation)
+                 | then([st = this->shared_from_this()](auto const & ec,
+                                                        const auto & /*msg*/) -> polymorphic_continuation_t<> {
+                       if (ec) {
+                           LOG_ERROR("Failed to send the configuration to the agent process: %s", ec.message().c_str());
+                           return st->cont_shutdown();
+                       }
+                       LOG_DEBUG("Write complete");
+                       return {};
+                   });
         }
 
         /** Handle the 'shutdown' IPC message variant. The agent is shutdown. */
@@ -184,20 +233,28 @@ namespace agents {
             }
         }
 
-        /** Handle the 'recv' IPC message variant. The agent received data from a connection. */
-        async::continuations::polymorphic_continuation_t<> cont_on_recv_message(
-            ipc::msg_annotation_recv_bytes_t && message)
+        /**
+         * Generic cont_on_recv_message() for when data should be sent to an APC.
+         * @tparam ReceiveMessageType Must have (1) a UID header and (2) a
+         * vector<uint8_t> suffix, this being the data to be written.
+         * @param message Message to write to APC directory.
+         */
+        template<typename ReceiveMessageType>
+        async::continuations::polymorphic_continuation_t<> cont_on_recv_message_send_to_apc(
+            ReceiveMessageType && message)
         {
             using namespace async::continuations;
 
-            LOG_DEBUG("Received ipc::msg_annotation_recv_bytes_t; uid=%d, size=%zu",
-                      message.header,
-                      message.suffix.size());
-
             auto uid = message.header;
+
+            if (closed_external_source_pipes.find(uid) != closed_external_source_pipes.end()) {
+                LOG_TRACE("Received data for external source with UID %d but already closed: doing nothing", uid);
+                return {};
+            }
+
             auto it = external_source_pipes.find(uid);
             if (it == external_source_pipes.end()) {
-                LOG_ERROR("Received data for external source but no pipe found");
+                LOG_ERROR("Received data for external source with UID %d but no pipe found", uid);
                 return {};
             }
 
@@ -224,21 +281,41 @@ namespace agents {
                    });
         }
 
+        /** Handle the 'recv' IPC message variant. The agent received data from a connection. */
+        async::continuations::polymorphic_continuation_t<> cont_on_recv_message(
+            ipc::msg_annotation_recv_bytes_t && message)
+        {
+            LOG_DEBUG("Received ipc::msg_annotation_recv_bytes_t; uid=%d, size=%zu",
+                      message.header,
+                      message.suffix.size());
+
+            return cont_on_recv_message_send_to_apc(message);
+        }
+
         /** Handle the 'close conn' IPC message variant. The agent closed a connection. */
         void cont_on_recv_message(ipc::msg_annotation_close_conn_t const & message)
         {
             LOG_DEBUG("Received ipc::msg_annotation_close_conn_t; uid=%d", message.header);
+            (void) close_external_source_pipe(message.header);
+        }
 
-            auto it = external_source_pipes.find(message.header);
-            if (it == external_source_pipes.end()) {
-                return;
-            }
+        /** Handle a received timeline message */
+        async::continuations::polymorphic_continuation_t<> cont_on_recv_message(ipc::msg_gpu_timeline_recv_t && message)
+        {
+            LOG_DEBUG("Received ipc::msg_gpu_timeline_recv_t; uid=%d, size=%zu", message.header, message.suffix.size());
 
-            // close the write end
-            it->second.close();
+            return cont_on_recv_message_send_to_apc(message);
+        }
 
-            // and remove from the map
-            external_source_pipes.erase(it);
+        /** Handle the handshake tag (ESTATE header) for timeline */
+        async::continuations::polymorphic_continuation_t<> cont_on_recv_message(
+            ipc::msg_gpu_timeline_handshake_tag_t && message)
+        {
+            LOG_DEBUG("Received ipc::msg_gpu_timeline_handshake_tag_t (ESTATE header); uid=%d, size=%zu",
+                      message.header,
+                      message.suffix.size());
+
+            return cont_on_recv_message_send_to_apc(message);
         }
 
         /**
@@ -262,6 +339,8 @@ namespace agents {
                                                      ipc::msg_shutdown_t,
                                                      ipc::msg_annotation_new_conn_t,
                                                      ipc::msg_annotation_recv_bytes_t,
+                                                     ipc::msg_gpu_timeline_handshake_tag_t,
+                                                     ipc::msg_gpu_timeline_recv_t,
                                                      ipc::msg_annotation_close_conn_t>(st->source_shared(),
                                                                                        use_continuation) //
                          | map_error()                                                                   //
@@ -279,10 +358,13 @@ namespace agents {
         ext_source_agent_worker_t(boost::asio::io_context & io_context,
                                   agent_process_t && agent_process,
                                   state_change_observer_t && state_change_observer,
-                                  ExternalSource & external_source)
+                                  ExternalSource & external_source,
+                                  ipc::msg_gpu_timeline_configuration_t gpu_timeline_message)
             : agent_worker_base_t(std::move(agent_process), std::move(state_change_observer)),
               strand(io_context),
-              external_source(external_source)
+              external_source(external_source),
+              gpu_timeline_config(gpu_timeline_message)
+
         {
         }
 
