@@ -1,8 +1,10 @@
-/* Copyright (C) 2023 by Arm Limited. All rights reserved. */
+/* Copyright (C) 2023-2024 by Arm Limited. All rights reserved. */
 
 #pragma once
 #include "lib/source_location.h"
 
+#include <cstdint>
+
 #include <sys/types.h>
 
 // This file contains those items/data that are arguments to the logging functions.
@@ -13,7 +15,7 @@
 
 namespace logging {
     /** Possible logging levels */
-    enum class log_level_t {
+    enum class log_level_t : uint8_t {
         trace,
         debug,
         setup,
@@ -37,5 +39,4 @@ namespace logging {
 
     /** Identifies the source thread */
     enum class thread_id_t : pid_t;
-
 }
